#ifndef PYTHONIC_INCLUDE_STRING_DIGITS_HPP
#define PYTHONIC_INCLUDE_STRING_DIGITS_HPP

#include "pythonic/types/str.hpp"

PYTHONIC_NS_BEGIN

namespace string
{
  types::str constexpr digits("0123456789");
}
PYTHONIC_NS_END

#endif
